@@ -1,11 +1,21 @@
 # sample from the posterior distribution of general linear panel
 # model in R using linked C++ code in Scythe
 #
+# NOTE: this does not take a data argument because only
+#       matrices are passed.  This should probably be fixed.  Also,
+#       re-implementing using something likes Bates' syntax
+#       would be nice.  The helper functions could also be used 
+#       all over the place here. This is another good project for a grad
+#       student.
+#
+#
 # ADM and KQ 8/1/2002
+# updated with Ben Goodrich's feedback and new spec ADM 7/28/2004
+
 
 "MCMCpanel" <-
   function(obs, Y, X, W, burnin = 1000, mcmc = 10000, thin = 5, 
-           verbose = FALSE, seed = 0, sigma2.start = NA,
+           verbose = FALSE, seed = NA, sigma2.start = NA,
            D.start = NA, b0 = 0, B0 = 1, eta0, R0, nu0 = 0.001,
            delta0 = 0.001, ...) {
 
@@ -50,6 +60,12 @@
     #      \varepsilon (k \times 1) vector of errors for subject i
     #
 
+    # seeds
+    seeds <- form.seeds(seed) 
+    lecuyer <- seeds[[1]]
+    seed.array <- seeds[[2]]
+    lecuyer.stream <- seeds[[3]]
+
     # model parameters
     n <- length(unique(obs))
     k <- length(Y) / n
@@ -57,29 +73,27 @@
     q <- dim(W)[2]
 
     # check data conformability
-    if(dim(obs)[2] != 1) {
-      cat("obs is not a column vector.\n")
-      stop("Please respecify and call MCMCpanel() again.\n")
+    obs.temp <- as.matrix(obs)
+    if (any(obs.temp[,1] != obs)) {
+      cat("Error: obs is not a column vector.\n")
+      stop("Please respecify and call MCMCpanel() again.\n", call.=FALSE)
     }   
     if(length(unique(tabulate(obs))) != 1) {
-      cat("Panel is not balanced [check obs vector].\n")
-      stop("Please respecify and call MCMCpanel() again.\n") 
+      cat("Error: Panel is not balanced [check obs vector].\n")
+      stop("Please respecify and call MCMCpanel() again.\n", call.=FALSE)
     }
-    if(dim(Y)[2] != 1) {
-      cat("Y is not a column vector.\n")
-      stop("Please respecify and call MCMCpanel() again.\n")
-    }
-    if(dim(X)[1] != n * k) {
-      cat("X matrix is not conformable [does not match Y].\n")
-      stop("Please respecify and call MCMCpanel() again.\n")  
+    Y.temp <- as.matrix(Y)
+    if (any(Y.temp[,1] != Y)) {
+      cat("Error: X matrix is not conformable [does not match Y].\n")
+      stop("Please respecify and call MCMCpanel() again.\n", call.=FALSE)
     }
     if(dim(W)[1] != n * k) {
-      cat("W matrix is not conformable [does not match Y].\n")
-      stop("Please respecify and call MCMCpanel() again.\n")
+      cat("Error: W matrix is not conformable [does not match Y].\n")
+      stop("Please respecify and call MCMCpanel() again.\n", call.=FALSE)
     }   
  
     # check iteration parameters
-    check.parameters(burnin, mcmc, thin, "MCMCpanel")
+    check.mcmc.parameters(burnin, mcmc, thin)
     totiter <- mcmc + burnin
 
     # starting values for beta error checking
@@ -95,8 +109,8 @@
       beta.start <- beta.start * matrix(1,p,1)  
     }
     if((dim(beta.start)[1] != p) || (dim(beta.start)[2] != 1)) {
-      cat("Starting value for beta not conformable.\n")
-      stop("Please respecify and call MCMCpanel() again.\n")
+      cat("Error: Starting value for beta not conformable.\n")
+      stop("Please respecify and call MCMCpanel() again.\n", call.=FALSE)
     }
   
     # sigma2 starting values error checking
@@ -104,8 +118,8 @@
       sigma2.start <- ols.sigma2
     }   
     if(sigma2.start <= 0) {
-      cat("Starting value for sigma2 negative.\n")
-      stop("Please respecify and call MCMCpanel() again.\n")
+      cat("Error: Starting value for sigma2 negative.\n")
+      stop("Please respecify and call MCMCpanel() again.\n", call.=FALSE)
     }   
 
     # starting values for D error checking
@@ -116,8 +130,8 @@
       D.start <- D.start * diag(q)
     }
     if((dim(D.start)[1] != q) || (dim(D.start)[2] != q)) {
-      cat("Starting value for D not conformable.\n")
-      stop("Please respecify and call MCMCpanel() again.\n")
+      cat("Error: Starting value for D not conformable.\n")
+      stop("Please respecify and call MCMCpanel() again.\n", call.=FALSE)
     }
 
     # set up prior for beta
@@ -125,38 +139,38 @@
       b0 <- b0 * matrix(1,p,1)  
     }
     if((dim(b0)[1] != p) || (dim(b0)[2] != 1)) {
-      cat("N(b0,B0^-1) prior b0 not conformable.\n")
-      stop("Please respecify and call MCMCpanel() again.\n")
+      cat("Error: N(b0,B0^-1) prior b0 not conformable.\n")
+      stop("Please respecify and call MCMCpanel() again.\n", call.=FALSE)
     }  
     if(is.null(dim(B0))) {
       B0 <- B0 * diag(p)
     }
     if((dim(B0)[1] != p) || (dim(B0)[2] != p)) {
-      cat("N(b0,B0^-1) prior B0 not conformable.\n")
-      stop("Please respecify and call MCMCpanel() again.\n")
+      cat("Error: N(b0,B0^-1) prior B0 not conformable.\n")
+      stop("Please respecify and call MCMCpanel() again.\n", call.=FALSE)
     } 
    
     # set up prior for sigma2
     if(nu0 <= 0) {
-      cat("G(nu0,delta0) prior nu0 less than or equal to zero.\n")
-      stop("Please respecify and call MCMCpanel() again.\n")
+      cat("Error: G(nu0,delta0) prior nu0 less than or equal to zero.\n")
+      stop("Please respecify and call MCMCpanel() again.\n", call.=FALSE)
     }
     if(delta0 <= 0) {
-      cat("G(nu0,delta0) prior delta0 less than or equal to zero.\n")
-      stop("Please respecify and call MCMCpanel() again.\n")      
+      cat("Error: G(nu0,delta0) prior delta0 less than or equal to zero.\n")
+      stop("Please respecify and call MCMCpanel() again.\n", call.=FALSE)
     }
        
     # set up prior for D
     if(eta0 < q) {
-      cat("Wishart(eta0,R0) prior eta0 less than or equal to q.\n")
-      stop("Please respecify and call MCMCpanel() again.\n")
+      cat("Error: Wishart(eta0,R0) prior eta0 less than or equal to q.\n")
+      stop("Please respecify and call MCMCpanel() again.\n", call.=FALSE)
     }   
     if(is.null(dim(R0))) {
       R0 <- R0 * diag(q)
     }
     if((dim(R0)[1] != q) || (dim(R0)[2] != q)) {
-      cat("Wishart(eta0,R0) prior R0 not comformable [q times q].\n")
-      stop("Please respecify and call MCMCpanel() again.\n")
+      cat("Error: Wishart(eta0,R0) prior R0 not comformable [q times q].\n")
+      stop("Please respecify and call MCMCpanel() again.\n", call.=FALSE)
     }
          
     # set up big holder matrix
@@ -168,11 +182,11 @@
                   samrow = as.integer(nrow(sample)),
                   samcol = as.integer(ncol(sample)),
                   obsdata = as.double(obs),
-                  obsrow = as.integer(nrow(obs)),
-                  obscol = as.integer(ncol(obs)),   
+                  obsrow = as.integer(length(obs)),
+                  obscol = as.integer(1),   
                   ydata = as.double(Y),
-                  yrow = as.integer(nrow(Y)),
-                  ycol = as.integer(ncol(Y)),   
+                  yrow = as.integer(length(Y)),
+                  ycol = as.integer(1),   
                   xdata = as.double(X),
                   xrow = as.integer(nrow(X)),
                   xcol = as.integer(ncol(X)),   
@@ -182,7 +196,9 @@
                   burnin = as.integer(burnin),
                   gibbs = as.integer(mcmc),
                   thin = as.integer(thin),
-                  seed = as.integer(seed),
+                  lecuyer = as.integer(lecuyer),
+                  seedarray = as.integer(seed.array),
+                  lecuyerstream = as.integer(lecuyer.stream),
                   verbose = as.integer(verbose),
                   bstartdata = as.double(beta.start),
                   bstartrow = as.integer(nrow(beta.start)),
@@ -214,11 +230,15 @@
     sample <- matrix(inv.obj$samdata, inv.obj$samrow, inv.obj$samcol,
                      byrow=TRUE)   
   
-    beta.names <- paste("beta", 1:p, sep = "")
+    if (length(colnames(X))>0) {
+        beta.names <- colnames(X)
+    }
+    else beta.names <- paste("beta", 1:p, sep = "")
     D.names <- paste("D", 1:(q*q), sep = "")
     sigma2.names <- "sigma2"
     names <- c(beta.names, D.names, sigma2.names)   
-    output <- mcmc2(data=sample, start=1, end=mcmc, thin=thin)
+
+    output <- mcmc(data=sample, start=1, end=mcmc, thin=thin)
     varnames(output) <- names
     attr(output,"title") <- 
       "MCMCpack Linear Panel Model Posterior Density Sample"
